from .generator import FullName
from .generator import FirstName
from .generator import LastName
from .generator import Image
from .generator import Date
from .generator import Text
from .generator import NumberedTitle
from .generator import EHTMLText
from .generator import Words
from .generator import Slug
from .generator import Constant
from .generator import Choice
from .generator import TrueOrFalse
from .generator import Relation

from .generator import Generator

