from ikcms.utils import cached_property
import ikcms.apps.base
import ikcms.cli.app


class App(ikcms.apps.base.App):

    commands = {
        '{{ name }}': ikcms.cli.app.AppCli,
    }

    def get_handler(self):
        from .handler import get_handler
        return get_handler(self)

    def get_env_class(self):
        from .env import Environment
        return Environment

